#ifndef EASY_CURL_H
#define EASY_CURL_H

#include <fstream>
#include <string>

#include <boost/function.hpp>

#include <curl/curl.h>

class easy_curl
{

public:

	easy_curl();
	~easy_curl();

	void set_fresh_connect(bool fresh);

	const std::string http_get(const std::string& url);
	void http_get_per_line(const std::string& url, const boost::function<void(const std::string& line)>& line_reader);

	const std::string ftp_list(const std::string& url);

	CURLcode download(const std::string& from, const std::string& to);
	CURLcode upload(const std::string& from, const std::string& to);

	CURLcode ftp_cmd(const std::string& url, const std::string& cmd);

	std::string format(const std::string& str);

	double get_speed() const;
	double get_time() const;
	long get_http_code() const;

	CURLcode get_err_no() const;
	std::string get_err_str() const;

private:

	CURL* m_curl;

	CURLcode m_error;

	std::string m_list;
	std::string m_contents;
	std::ifstream m_infile;
	std::ofstream m_outfile;

	boost::function<void (const std::string& line)> m_line_reader;  

	long m_http_code;
	double m_speed;
	double m_time;

	bool m_fresh_connect;

	void reset();
  
	easy_curl(const easy_curl& ftp);
	easy_curl& operator=(const easy_curl& ftp);

	static size_t contents_builder(void* buffer, size_t size, size_t bytes, void* object);
	static size_t data_writer(void* buffer, size_t size, size_t bytes, void* object);
	static size_t data_reader(void* buffer, size_t size, size_t bytes, void* object);
	static size_t list_builder(void* buffer, size_t size, size_t bytes, void* object);

};

#endif

