#include <iostream>
#include <vector>
#include <sstream>
#include <string>

#include <boost/thread.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string/case_conv.hpp>

#include <curl/curl.h>

#include "easy_curl.h"
#include "ftpparse.h"

// EDIT ZONE START
const std::string serv1 = "ftp://user:pass@server.com/dir1";
const std::string serv2 = "ftp://user:pass@server.com/dir2";
const std::string file_to_delete1 = "a.txt";
const std::string file_to_delete2 = "a.txt";
// EDIT ZONE END

struct fileinfo
{
	std::string path;
	std::string name;
	std::string size;
	std::string chmod;
	std::string is_dir;

	bool operator<(const fileinfo& file) const
	{
		if(is_dir == file.is_dir)
			return (boost::algorithm::to_lower_copy(name) < boost::algorithm::to_lower_copy(file.name));
		else
			return (is_dir == "1");    
	}
};

void recurse_ftp_dir(easy_curl& curl, const std::string& ftp_url, const std::string& ftp_path)
{
	// Build ftp path
	std::string full_path = ftp_url;
	if(!ftp_path.empty())
		full_path += "/" + ftp_path;

	// Get ftp list
	std::string list = curl.ftp_list(full_path + "/");

	// Parse list line per line
	std::string line;
	std::istringstream ss(list);
	std::vector<fileinfo> files;
	while(getline(ss, line))
	{
		struct ftpparse p;
		if(!ftpparse(&p, line.c_str(), line.length()))
			std::cerr << __FILE__ ": skipped " << line << std::endl << std::endl;
		else
		{
			// Skip what we don't want & store file info
			std::string name(p.name, p.name + p.namelen - 1);
			if(name != "." && name != "..")
			{
				bool is_dir = p.flagtrycwd;

				fileinfo file;
				file.path = ftp_path;
				file.name = name;
				file.size = ((is_dir) ? "0" : boost::lexical_cast<std::string>(p.size));
				file.chmod = p.chmod;
				file.is_dir = (is_dir) ? "1" : "0";

				files.push_back(file);
			}
		}
	}

	// Sort the files (dir 1st + alpha sorted)
	std::sort(files.begin(), files.end());   

	// Save file info & go look into directories
	for(std::vector<fileinfo>::iterator it = files.begin(); it != files.end(); ++it)
	{
		if(it->is_dir == "1")
			recurse_ftp_dir(curl, ftp_url, (it->path.empty()) ? it->name : it->path + "/" + it->name);
	}
}

struct thread_t
{
	std::string ftp_url;
	std::string file;

	thread_t(const std::string& url, const std::string todel) : ftp_url(url), file(todel)
	{
	}

	void operator()()
	{
		easy_curl curl;

		// Get files info
		recurse_ftp_dir(curl, ftp_url, "");

		// Remove file
		if(curl.ftp_cmd(ftp_url + "/", "DELE " + file) != CURLE_OK)
			std::cout << "Error during ftp delete " << curl.get_err_no() << std::endl;
	}
};

int main(int argc, char* argv[])
{
	// Initializes curl
	curl_global_init(CURL_GLOBAL_ALL);

	// Run threads
	boost::thread t(thread_t(serv1, file_to_delete1));
	boost::thread t2(thread_t(serv2, file_to_delete2));
	t.join();
	t2.join();

	// Closes curl
	curl_global_cleanup();

	return 0;
}
